<?php

declare(strict_types = 1);

namespace Drupal\config_revision\Controller;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\Controller\RevisionOverviewController as CoreRevisionOverviewController;

/**
 * Revision overview controller class.
 */
class RevisionOverviewController extends CoreRevisionOverviewController {

  /**
   * {@inheritdoc}
   */
  protected function revisionOverview(ContentEntityInterface $entity) {
    $cacheability = (new CacheableMetadata())
      ->addCacheableDependency($entity);
    $entity_storage = $this->entityTypeManager()
      ->getStorage($entity->getEntityTypeId());

    $header = [$this->t('Revision'), $this->t('Operations')];
    $rows = [];

    $entity_revisions = $entity_storage->loadMultipleRevisions($this->revisionIds($entity));

    foreach ($entity_revisions as $revision) {
      $row = [];
      /** @var \Drupal\Core\Entity\ContentEntityInterface $revision */
      $row[] = $this->getRevisionDescription($revision, $revision->isDefaultRevision());

      if ($revision->isDefaultRevision()) {
        $row[] = [
          'data' => [
            '#prefix' => '<em>',
            '#markup' => $this->t('Current revision'),
            '#suffix' => '</em>',
          ],
        ];
        foreach ($row as &$current) {
          $current['class'] = ['revision-current'];
        }
      }
      else {
        $links = $this->getOperationLinks($revision);
        $row[] = [
          'data' => [
            '#type' => 'operations',
            '#links' => $links,
          ],
        ];
      }

      $rows[] = $row;
    }

    $build['entity_revisions_table'] = [
      '#theme' => 'table',
      '#rows' => $rows,
      '#header' => $header,
    ];

    $cacheability->applyTo($build);

    return $build;
  }

}
